/*****************************************************************************/
/*                                                                           */
/*  ****                      ESS Technology                            **** */
/*                                                                           */
/*                                                                           */
/*                        MPU401 Interface Library                           */
/*                                                                           */
/*                                06/17/96                                   */
/*                                                                           */
/*  ****                                                                **** */
/*                                                                           */
/*****************************************************************************/
#include	<stdio.h>
#include	<stdlib.h>
#include	<conio.h>
#include	<dos.h>

#include	"stdtype.h"
#include    "mpu401.h"
#include	"delay.h"

/*****************************************************************************/
/* bGetMPU401Status - Return MPU401 I/O Port Status                          */
/*                                                                           */
/*****************************************************************************/
BYTE    bGetMPU401Status( void )
	{
	return ( ( BYTE ) inp( _ES_MPU401_STATUS ) );
	}

/*****************************************************************************/
/* bGetMPU401RdStatus - Return MPU401 I/O Port Read Status                   */
/*                                                                           */
/*****************************************************************************/
BYTE    bGetMPU401RdStatus( void )
	{
	return ( ( BYTE ) ( ( ( bGetMPU401Status(  ) & _ES_MPU401_RD_MSK ) == ( BYTE ) _ES_MPU401_RD_RDY ) ? 1 : 0 ) );
	}

/*****************************************************************************/
/* bGetMPU401WrStatus - Return MPU401 I/O Port Write Status                  */
/*                                                                           */
/*****************************************************************************/
BYTE    bGetMPU401WrStatus( void )
	{
	return ( ( BYTE ) ( ( ( bGetMPU401Status(  ) & _ES_MPU401_WR_MSK ) == ( BYTE ) _ES_MPU401_WR_RDY ) ? 1 : 0 ) );
	}

/*****************************************************************************/
/* iWrMPU401Cmd - Write Command to MPU401 I/O Port                           */
/*                                                                           */
/*****************************************************************************/
int     iWrMPU401Cmd( BYTE _bMPU401Cmd )
	{
	int     iRetVal = _ES_MPU401_ERROR;
	
	BYTE	bTimeoutFlag = FALSE;
    
    vSetEventWait( _ES_MPU401_RD_TIMEOUT, &bTimeoutFlag );
    
	while( bTimeoutFlag == FALSE )
		if ( bGetMPU401WrStatus(  ) )
			{
			iRetVal = outp( _ES_MPU401_CMD, _bMPU401Cmd );
			break;
			}

	vClearEventWait();
	
	return ( iRetVal );
	}

/*****************************************************************************/
/* iWrMPU401Data - Write Data to MPU401 I/O Port                             */
/*                                                                           */
/*****************************************************************************/
int     iWrMPU401Data( BYTE _bMPU401Data )
	{
	int     iRetVal = _ES_MPU401_ERROR;

	BYTE	bTimeoutFlag = FALSE;
	
	vSetEventWait( _ES_MPU401_WR_TIMEOUT, &bTimeoutFlag );
	
	while( bTimeoutFlag == FALSE )
		{
		if ( bGetMPU401WrStatus(  ) )
			{
			iRetVal = outp( _ES_MPU401_DATA, _bMPU401Data );
			break;
			}
		}
        
	vClearEventWait();
	        
	return ( iRetVal );
	}

/*****************************************************************************/
/* iRdMPU401Data - Read Data from MPU401 I/O Port                            */
/*                                                                           */
/*****************************************************************************/
int     iRdMPU401Data( void )
	{
	int     iRetVal = _ES_MPU401_ERROR;
	
	BYTE	bTimeoutFlag = FALSE;
	
	vSetEventWait( _ES_MPU401_RD_TIMEOUT, &bTimeoutFlag );
	
	while( bTimeoutFlag == FALSE )
		{
		if ( bGetMPU401RdStatus(  ) )
			{
			iRetVal = inp( _ES_MPU401_DATA );
			break;
			}
		}
    
    vClearEventWait();
    
	return ( iRetVal );
	}

/*****************************************************************************/
/* iResetMPU401 - Reset MPU401 I/O Port                                      */
/*                                                                           */
/*****************************************************************************/
int     iResetMPU401( void )
	{
	int     iRetVal = _ES_MPU401_ERROR,
	        iTimeout = _ES_MPU401_TIMEOUT;	/* Number of tries */
    
	BYTE	bTimeoutFlag = FALSE;
	
	vSetEventWait( _ES_MPU401_TIMEOUT, &bTimeoutFlag );
	
	while ( bTimeoutFlag == FALSE )
		{
		if ( iWrMPU401Cmd( _ES_MPU401_RESET ) != _ES_MPU401_ERROR )
			{
			if ( iRdMPU401Data(  ) == _ES_MPU401_SMART_RDY )
				{
				iRetVal = _ES_MPU401_OK;
				break;
				}
			else
				{
				}
			}
		else
			{
			}
		}

	vClearEventWait();
	
	return ( iRetVal );
	}

/*****************************************************************************/
/* iEnableSmartMode - Enable MPU401 Smart Mode                               */
/*                                                                           */
/*****************************************************************************/
int     iEnableSmartMode( void )
	{
	int     iRetVal = _ES_MPU401_ERROR,
	        iTimeout = _ES_MPU401_TIMEOUT;	/* Number of tries */

	while ( --iTimeout )
		{
		iWrMPU401Cmd( _ES_MPU401_SMART_MODE );
		if ( iRdMPU401Data(  ) == _ES_MPU401_SMART_RDY )
			{
			iRetVal = _ES_MPU401_OK;
			break;
			}
		}

	return ( iRetVal );
	}

/*****************************************************************************/
/* iEnableMPU401UartMode - Enable MPU401 UART Mode                           */
/*                                                                           */
/*****************************************************************************/
int     iEnableUartMode( void )
	{
	int     iRetVal = _ES_MPU401_ERROR,
	        iTimeout = _ES_MPU401_TIMEOUT;	/* Number of tries */

	while ( --iTimeout )
		{
		if ( iWrMPU401Cmd( _ES_MPU401_UART_MODE ) != _ES_MPU401_ERROR )
			if ( iRdMPU401Data(  ) == _ES_MPU401_UART_RDY )
				{
				iRetVal = _ES_MPU401_OK;
				break;
				}
		}

	return ( iRetVal );
	}

/*****************************************************************************/
/* iSendSysExMessage - Send System Exclusive Message to MPU401 I/O Port      */
/*                                                                           */
/*****************************************************************************/
int     iSendSysExMessage( LPBYTE _pbSysExMsg )
	{
	BYTE    bIndex = 0;

	do
		{
		if ( iWrMPU401Data( _pbSysExMsg[bIndex] ) == _ES_MPU401_ERROR )
			{
			return ( _ES_MPU401_ERROR );
			}
		}

	while ( _pbSysExMsg[bIndex++] != 0xF7 );

	return ( _ES_MPU401_OK );
	}

/*****************************************************************************/
/* iReadSysExMessage - Read System Exclusive Message from MPU401 I/O Port    */
/*                                                                           */
/*****************************************************************************/
int     iReadSysExMessage( LPBYTE _pbSysExMsg )
	{
	int     iTmpData;

	BYTE    bIndex = 0;

	do
		{
		if ( ( iTmpData = iRdMPU401Data(  ) ) == _ES_MPU401_ERROR )
			{
			return ( _ES_MPU401_ERROR );
			}

		_pbSysExMsg[bIndex] = ( BYTE ) iTmpData;
		}

	while ( _pbSysExMsg[bIndex++] != 0xF7 );

	return ( _ES_MPU401_OK );
	}

